"""supporting functions"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/utils.ipynb.

# %% auto 0
__all__ = ['printmd']

# %% ../../nbs/utils/utils.ipynb 3
from .convert import *
from .files import *

# %% ../../nbs/utils/utils.ipynb 4
from IPython.display import Markdown, display

# %% ../../nbs/utils/utils.ipynb 5
def printmd(string):
    display(Markdown(string))
